/*

    umesh2dgui, a GUI for umesh2d grid generator

    Copyright (C) 2011 Gabriele Dini Ciacci <dark.schneider@iol.it>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   Created on June 9, 2011, 11:05 PM
 */

#ifndef U2D_DISPLAYTEXT_H
#define	U2D_DISPLAYTEXT_H

#include <string>
#include <FTGL/ftgl.h>
#include <boost/shared_ptr.hpp>

#include "gl/u2d_DisplayObject.h"


class u2d_DisplayText : public u2d_DisplayObject {
public:
    u2d_DisplayText(std::string _name);
    virtual ~u2d_DisplayText();
    
    void draw();

    std::string getName() const;

    bool isEnabled() const;
    
    u2d_DisplayText &enable();
    u2d_DisplayText &disable();
    u2d_DisplayText &setFontSize(GLushort size, GLushort dpi = 100);
    u2d_DisplayText &setSize(GLfloat _size);
    u2d_DisplayText &setPosition(GLfloat x, GLfloat y, GLfloat z = 0.0);
    u2d_DisplayText &setOffset(GLfloat x, GLfloat y, GLfloat z = 0.0);
    u2d_DisplayText &setColor(GLfloat r, GLfloat g, GLfloat b, GLfloat a = 1.0);
    u2d_DisplayText &setText(std::string _text);
    template <class T> u2d_DisplayText &operator<<(T);
    
private:
    GLushort fontSize;
    boost::shared_ptr<FTFont> font;
    boost::shared_ptr<FTPoint> offset;
    std::string name;
    bool enabled;
    std::stringstream text;
    GLfloat scaleFactor[3];
    GLfloat position[3];
    GLfloat color[4];
    
    
    

};

template <class T> u2d_DisplayText &u2d_DisplayText::operator<<(T _text) {
    this->text << _text;
    return *this;
}

#endif	/* U2D_DISPLAYTEXT_H */

