/*

    umesh2dgui, a GUI for umesh2d grid generator

    Copyright (C) 2010 Gabriele Dini Ciacci <dark.schneider@iol.it>
    Based on a code from Luigi Quartapelle and Massimo Biava

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Created on February 22, 2010, 12:34 PM
 */

#ifndef _U2D_CONSTANTS_H
#define	_U2D_CONSTANTS_H

#include "boost/numeric/mtl/mtl.hpp"

using namespace mtl;

/**
 * Utility class that gathers all functions that generate the various
 * constants. Scalar, vector or fields.
 */
class u2d_Constants {
public:
    u2d_Constants();
    u2d_Constants(const u2d_Constants& orig);
    virtual ~u2d_Constants();
    static dense_vector<double> thermalConductivity(dense2D<double> rr);
    static dense2D<double> gradThermalConductivity(dense2D<double> rr);
    static dense2D<double> advection(dense2D<double> rr);
    static dense_vector<double> pythagoras(dense2D<double> rr);
    static dense2D<double> gradPythagoras(dense2D<double> rr);
    static dense_vector<double> laplacianSolution(dense2D<double> rr);
    static dense_vector<double> source(dense2D<double> rr, double gamma);
    static void print(dense2D<double> matrix);
    static void print(dense_vector<double> vector);
    static void compare(dense_vector<double> a, dense_vector<double> b);
    static void compare(dense2D<double> a, dense2D<double> b);
private:

};

#endif	/* _U2D_CONSTANTS_H */

