/*

    umesh2dgui, a GUI for umesh2d grid generator

    Copyright (C) 2010 Gabriele Dini Ciacci <dark.schneider@iol.it>
    Based on a code from Luigi Quartapelle and Massimo Biava

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Created on January 27, 2010, 6:10 PM
 */

#ifndef _U2D_PROBLEMCALCULATOR_H
#define	_U2D_PROBLEMCALCULATOR_H

#include "u2d_Problem.h"
#include "u2d_DirichletCondition.h"
#include "u2d_GaussPoints.h"
#include "boost/numeric/mtl/mtl.hpp"
#include "u2d_NeumannCondition.h"

using namespace mtl;

class u2d_ProblemCalculator {
public:
    u2d_ProblemCalculator(u2d_Problem *_problem, u2d_Mesh *_mesh);
    u2d_ProblemCalculator(const u2d_ProblemCalculator& orig);
    virtual ~u2d_ProblemCalculator();

    /* The various term calculators */
    /* LHS terms */
    void forceDirichlet_left(u2d_DirichletCondition *cond);
    void addAlphaW__G_left(u2d_DomainGaussPoints *dgp, double alpha);
    void addDW__AlphaDG_left(u2d_DomainGaussPoints *dgp, dense_vector<double> alpha);
    void addW__AlphaDG_left(u2d_DomainGaussPoints *dgp, dense2D<double> alpha);
    void addAlphaW__G_surf_left(u2d_BoundaryGaussPoints *bgp, u2d_NeumannCondition *cond, double alpha);
    /* RHS terms */
    void forceDirichlet_right(u2d_DirichletCondition *cond, dense_vector<double> sol);
    void addW__G_right(u2d_DomainGaussPoints *dgp, dense_vector<double> source);
    void addW__G_surf_right(u2d_BoundaryGaussPoints *bgp, u2d_NeumannCondition *nc, dense_vector<double> sol);
    void addW__divG_surf_right(u2d_BoundaryGaussPoints *bgp, u2d_NeumannCondition *cond, dense2D<double> field);
private:
    u2d_Problem *problem;
    u2d_Mesh *mesh;


};

#endif	/* _U2D_PROBLEMCALCULATOR_H */

